@@ -1,3 +1,10 @@
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
+#include <DGuiApplicationHelper>
+#include <DFontManager>
+
 #include <QApplication>
 #include <QDesktopWidget>
 
@@ -29,5 +36,10 @@ int main(int argc, char *argv[])
         QFile::remove(desktopFile.fileName());
     });
 
+    //控制中心修改字体大小可以看到打印输出
+    QObject::connect(DGuiApplicationHelper::instance()->fontManager(), &DFontManager::fontChanged, [] {
+        qDebug() << DGuiApplicationHelper::instance()->fontManager()->baseFont();
+    });
+
     return a.exec();
 }
