@@ -1,27 +1,11 @@
-/*
- * Copyright (C) 2019 ~ 2019 Deepin Technology Co., Ltd.
- *
- * Author:     zccrs <zccrs@live.com>
- *
- * Maintainer: zccrs <zhangjide@deepin.com>
- *
- * This program is free software: you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2022 UnionTech Software Technology Co., Ltd.
+//
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
 #include "dguiapplicationhelper.h"
 #include "private/dguiapplicationhelper_p.h"
 #include "dplatformhandle.h"
-#include <util/DFontManager>
+#include <DFontManager>
 
 #include <QHash>
 #include <QColor>
@@ -52,6 +36,7 @@
 #include <QLockFile>
 #include <QDirIterator>
 #include <QDesktopServices>
+#include <DStandardPaths>
 
 #ifdef Q_OS_UNIX
 #include <QDBusError>
@@ -61,6 +46,7 @@
 #include <QDBusConnection>
 #include <QDBusConnectionInterface>
 #include <QProcess>
+#include <DPathBuf>
 #endif
 
 #include <private/qguiapplication_p.h>
@@ -120,6 +106,9 @@ Q_GLOBAL_STATIC(DFontManager, _globalFM)
 
 #define WINDOW_THEME_KEY "_d_platform_theme"
 
+/*!
+ @private
+ */
 class _DGuiApplicationHelper
 {
 public:
@@ -150,27 +139,59 @@ public:
     static DGuiApplicationHelper::HelperCreator creator;
 };
 
+class LoadManualServiceWorker : public QThread
+{
+public:
+    explicit LoadManualServiceWorker(QObject *parent = nullptr);
+    ~LoadManualServiceWorker() override;
+    void checkManualServiceWakeUp();
+
+protected:
+    void run() override;
+};
+
+LoadManualServiceWorker::LoadManualServiceWorker(QObject *parent)
+    : QThread(parent)
+{
+    if (!parent)
+        connect(qApp, &QGuiApplication::aboutToQuit, this, std::bind(&LoadManualServiceWorker::exit, this, 0));
+}
+
+LoadManualServiceWorker::~LoadManualServiceWorker()
+{
+}
+
+void LoadManualServiceWorker::run()
+{
+    QDBusInterface("com.deepin.Manual.Search",
+                   "/com/deepin/Manual/Search",
+                   "com.deepin.Manual.Search");
+}
+
+void LoadManualServiceWorker::checkManualServiceWakeUp()
+{
+    if (this->isRunning())
+        return;
+
+    start();
+}
+
 DGuiApplicationHelper::HelperCreator _DGuiApplicationHelper::creator = _DGuiApplicationHelper::defaultCreator;
 Q_GLOBAL_STATIC(_DGuiApplicationHelper, _globalHelper)
 
 int DGuiApplicationHelperPrivate::waitTime = 3000;
 DGuiApplicationHelper::Attributes DGuiApplicationHelperPrivate::attributes = DGuiApplicationHelper::UseInactiveColorGroup;
+static const DGuiApplicationHelper::SizeMode InvalidSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(-1);
 
 DGuiApplicationHelperPrivate::DGuiApplicationHelperPrivate(DGuiApplicationHelper *qq)
     : DObjectPrivate(qq)
+    , explicitSizeMode(InvalidSizeMode)
 {
 
 }
 
 void DGuiApplicationHelperPrivate::init()
 {
-    D_Q(DGuiApplicationHelper);
-
-    systemTheme = new DPlatformTheme(0, q);
-    // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
-    // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
-    appTheme = systemTheme;
-
     if (qGuiApp) {
         initApplication(qGuiApp);
     } else {
@@ -184,6 +205,15 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
 {
     D_Q(DGuiApplicationHelper);
 
+    if (!systemTheme) {
+        // 需要在QGuiApplication创建后再创建DPlatformTheme，否则DPlatformTheme无效.
+        // qGuiApp->platformFunction()会报警告，并返回nullptr.
+        systemTheme = new DPlatformTheme(0, q);
+        // 直接对应到系统级别的主题, 不再对外提供为某个单独程序设置主题的接口.
+        // 程序设置自身主题相关的东西皆可通过 setPaletteType 和 setApplicationPalette 实现.
+        appTheme = systemTheme;
+    }
+
     // 跟随application销毁
     qAddPostRoutine(staticCleanApplication);
 
@@ -211,6 +241,9 @@ void DGuiApplicationHelperPrivate::initApplication(QGuiApplication *app)
         // 此时appTheme等价于systemTheme, 可以直接信号链接
         _q_initApplicationTheme();
     }
+
+    systemSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(systemTheme->sizeMode());
+    q->connect(systemTheme, SIGNAL(sizeModeChanged(int)), q, SLOT(_q_sizeModeChanged(int)));
 }
 
 void DGuiApplicationHelperPrivate::staticInitApplication()
@@ -218,11 +251,8 @@ void DGuiApplicationHelperPrivate::staticInitApplication()
     if (!_globalHelper.exists())
         return;
 
-    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load()) {
-        // systemTheme未创建时说明DGuiApplicationHelper还未初始化
-        if (helper->d_func()->systemTheme)
-            helper->d_func()->initApplication(qGuiApp);
-    }
+    if (DGuiApplicationHelper *helper = _globalHelper->m_helper.load())
+        helper->d_func()->initApplication(qGuiApp);
 }
 
 void DGuiApplicationHelperPrivate::staticCleanApplication()
@@ -303,6 +333,40 @@ bool DGuiApplicationHelperPrivate::isCustomPalette() const
     return appPalette || paletteType != DGuiApplicationHelper::UnknownType;
 }
 
+void DGuiApplicationHelperPrivate::_q_sizeModeChanged(int mode)
+{
+    D_Q(DGuiApplicationHelper);
+    qCInfo(dgAppHelper) << "Receiving that system size mode is set to [" << static_cast<DGuiApplicationHelper::SizeMode>(mode)
+                        << "], and old system size mode is [" << systemSizeMode << "]";
+
+    const auto oldSizeMode = fetchSizeMode();
+    systemSizeMode = static_cast<DGuiApplicationHelper::SizeMode>(mode);
+    const auto currentSizeMode = fetchSizeMode();
+    if (oldSizeMode != currentSizeMode)
+        Q_EMIT q->sizeModeChanged(currentSizeMode);
+}
+
+DGuiApplicationHelper::SizeMode DGuiApplicationHelperPrivate::fetchSizeMode(bool *isSystemSizeMode) const
+{
+    if (isSystemSizeMode)
+        *isSystemSizeMode = false;
+    // `setSizeMode` > `D_DTK_SIZEMODE` > `systemSizeMode`
+    if (explicitSizeMode != InvalidSizeMode)
+        return explicitSizeMode;
+
+    static const QString envSizeMode(qEnvironmentVariable("D_DTK_SIZEMODE"));
+    if (!envSizeMode.isEmpty()) {
+        bool ok = false;
+        const auto mode = envSizeMode.toInt(&ok);
+        if (ok)
+            return static_cast<DGuiApplicationHelper::SizeMode>(mode);
+    }
+
+    if (isSystemSizeMode)
+        *isSystemSizeMode = true;
+    return systemSizeMode;
+}
+
 /*!
   \class Dtk::Gui::DGuiApplicationHelper
   \inmodule dtkgui
@@ -310,45 +374,45 @@ bool DGuiApplicationHelperPrivate::isCustomPalette() const
  */
 
 /*!
-  \enum Dtk::Gui::DGuiApplicationHelper::ColorType
+  \enum DGuiApplicationHelper::ColorType
   DGuiApplicationHelper::ColorType 定义了主题类型.
   
-  \value UnknownType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::UnknownType
   未知主题(浅色主题或深色主题)
   
-  \value LightType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::LightType
   浅色主题
   
-  \value DarkType
+  \var DGuiApplicationHelper::ColorType DGuiApplicationHelper::DarkType
   深色主题
  */
 
 /*!
-  \enum Dtk::Gui::DGuiApplicationHelper::Attribute
+  \enum DGuiApplicationHelper::Attribute
   DGuiApplicationHelper::Attribute 定义了功能属性
   
-  \value UseInactiveColorGroup
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::UseInactiveColorGroup
   如果开启，当窗口处于Inactive状态时就会使用QPalette::Inactive的颜色，否则窗口将没有任何颜色变化。
   
-  \value ColorCompositing
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::ColorCompositing
   是否采用半透明样式的调色板。
   
-  \value ReadOnlyLimit
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::ReadOnlyLimit
   区分只读枚举。
   
-  \value IsDeepinPlatformTheme
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDeepinPlatformTheme
   获取当前是否使用deepin的platformtheme插件，platformtheme插件可以为Qt程序提供特定的控件样式，默认使用chameleon主题。
   
-  \value IsDXcbPlatform
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDXcbPlatform
   获取当前使用的是不是dtk的xcb窗口插件，dxcb插件提供了窗口圆角和阴影功能。
   
-  \value IsXWindowPlatform
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsXWindowPlatform
   获取当前是否运行在X11环境中。
   
-  \value IsTableEnvironment
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsTableEnvironment
   获取当前是否运行在deepin平板环境中，检测XDG_CURRENT_DESKTOP环境变量是不是tablet结尾。
   
-  \value IsDeepinEnvironment
+  \var DGuiApplicationHelper::Attribute DGuiApplicationHelper::IsDeepinEnvironment
   获取当前是否运行在deepin桌面环境中，检测XDG_CURRENT_DESKTOP环境变量是不是deepin。
  */
 
@@ -369,7 +433,7 @@ void DGuiApplicationHelper::initialize()
 /*!
   \brief 创建 DGuiApplicationHelper 对象.
 
-  \a creator 函数指针
+  \param creator 函数指针
   \note 一定要先调用此函数,再使用 DGuiApplicationHelper::instance()
  */
 void DGuiApplicationHelper::registerInstanceCreator(DGuiApplicationHelper::HelperCreator creator)
@@ -409,14 +473,14 @@ DGuiApplicationHelper::~DGuiApplicationHelper()
 
   \note 取值范围均为 -100 ~ 100 ,当三原色参数为-100时，颜色为黑色，参数为100时，颜色为白色.
   以透明度( alphaFloat )为例,当参数为负数时基础色的 alphaFloat 值减少，现象偏向透明, 参数为正数alphaFloat 值增加，现象偏不透明
-  \a base 基础色
-  \a hueFloat 色调
-  \a saturationFloat 饱和度
-  \a lightnessFloat 亮度
-  \a redFloat 红色
-  \a greenFloat 绿色
-  \a blueFloat 蓝色
-  \a alphaFloat Alpha通道(透明度)
+  \param base 基础色
+  \param hueFloat 色调
+  \param saturationFloat 饱和度
+  \param lightnessFloat 亮度
+  \param redFloat 红色
+  \param greenFloat 绿色
+  \param blueFloat 蓝色
+  \param alphaFloat Alpha通道(透明度)
   \return 经过调整的颜色
  */
 QColor DGuiApplicationHelper::adjustColor(const QColor &base,
@@ -452,14 +516,14 @@ QColor DGuiApplicationHelper::adjustColor(const QColor &base,
 
   \note 取值范围均为 -100 ~ 100 ,当三原色参数为-100时，颜色为黑色，参数为100时，颜色为白色.
   以透明度( alphaFloat )为例,当参数为负数时基础色的 alphaFloat 值减少，现象偏向透明, 参数为正数alphaFloat 值增加，现象偏不透明
-  \a base 基础色
-  \a hueFloat 色调
-  \a saturationFloat 饱和度
-  \a lightnessFloat 亮度
-  \a redFloat 红色
-  \a greenFloat 绿色
-  \a blueFloat 蓝色
-  \a alphaFloat Alpha通道(透明度)
+  \param base 基础色
+  \param hueFloat 色调
+  \param saturationFloat 饱和度
+  \param lightnessFloat 亮度
+  \param redFloat 红色
+  \param greenFloat 绿色
+  \param blueFloat 蓝色
+  \param alphaFloat Alpha通道(透明度)
   \return 经过调整的图片
  */
 QImage DGuiApplicationHelper::adjustColor(const QImage &base, qint8 hueFloat, qint8 saturationFloat, qint8 lightnessFloat, qint8 redFloat, qint8 greenFloat, qint8 blueFloat, qint8 alphaFloat)
@@ -489,8 +553,8 @@ QImage DGuiApplicationHelper::adjustColor(const QImage &base, qint8 hueFloat, qi
 /*!
   \brief 将两种颜色混合，合成新的颜色.
 
-  \a substrate 底层颜色
-  \a superstratum 上层颜色
+  \param substrate 底层颜色
+  \param superstratum 上层颜色
   \return 混合颜色
  */
 QColor DGuiApplicationHelper::blendColor(const QColor &substrate, const QColor &superstratum)
@@ -589,7 +653,7 @@ static QColor dark_dpalette[DPalette::NColorTypes] {
 /*!
   \brief 根据主题获取标准调色板.
 
-  \a type 主题枚举值
+  \param type 主题枚举值
   \return 调色板
  */
 DPalette DGuiApplicationHelper::standardPalette(DGuiApplicationHelper::ColorType type)
@@ -757,9 +821,9 @@ static void generatePaletteColor_helper(DPalette &base, M role, DGuiApplicationH
 /*!
   \brief 获取调色板颜色.
 
-  \a base 调色板
-  \a role 色码
-  \a type 主题枚举值
+  \param base 调色板
+  \param role 色码
+  \param type 主题枚举值
   \sa QPalette::ColorRole
  */
 void DGuiApplicationHelper::generatePaletteColor(DPalette &base, QPalette::ColorRole role, DGuiApplicationHelper::ColorType type)
@@ -793,9 +857,9 @@ void DGuiApplicationHelper::generatePaletteColor(DPalette &base, QPalette::Color
   0.7 后作为 QPalette::Disabled 类的颜色使用, 调整为 0.6 后作为 QPalette::Inactive
   类的颜色使用.
 
-  \a base 被加工的调色板
-  \a role 加工的项
-  \a type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
+  \param base 被加工的调色板
+  \param role 加工的项
+  \param type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
  */
 void DGuiApplicationHelper::generatePaletteColor(DPalette &base, DPalette::ColorType role, DGuiApplicationHelper::ColorType type)
 {
@@ -809,8 +873,8 @@ void DGuiApplicationHelper::generatePaletteColor(DPalette &base, DPalette::Color
   同 generatePaletteColor, 将直接调用 generatePaletteColor 加工
   所有类型的调色板颜色.
 
-  \a base 被加工的调色板
-  \a type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
+  \param base 被加工的调色板
+  \param type 加工时所使用的颜色类型, 如果值为 UnknownType 将使用 toColorType 获取颜色类型
  */
 void DGuiApplicationHelper::generatePalette(DPalette &base, ColorType type)
 {
@@ -839,7 +903,7 @@ void DGuiApplicationHelper::generatePalette(DPalette &base, ColorType type)
   根据颜色类型将使用 standardPalette 获取基础调色板数据, 在此基础上
   从 DPlatformTheme::fetchPalette 获取最终的调色板.
 
-  \a theme 平台主题对象
+  \param theme 平台主题对象
   \return 调色板数据
  */
 DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
@@ -877,7 +941,7 @@ DPalette DGuiApplicationHelper::fetchPalette(const DPlatformTheme *theme)
   \brief 设置是否将调色板的颜色改为半透明模式.
 
   一般用在主窗口背景为透明、模糊的程序中
-  \a on 是否开启
+  \param on 是否开启
  */
 void DGuiApplicationHelper::setUseInactiveColorGroup(bool on)
 {
@@ -887,7 +951,7 @@ void DGuiApplicationHelper::setUseInactiveColorGroup(bool on)
 /*!
   \brief 设置是否开启混合颜色.
 
-  \a on 是否开启
+  \param on 是否开启
  */
 void DGuiApplicationHelper::setColorCompositingEnabled(bool on)
 {
@@ -957,7 +1021,7 @@ DPlatformTheme *DGuiApplicationHelper::applicationTheme() const
 /*!
   \brief 返回窗口级别的主题, 优先级高于 windowTheme 和 systemTheme.
 
-  \a window 主题对象对应的窗口
+  \param window 主题对象对应的窗口
   \return 平台主题对象
   \sa applicationTheme()
   \warning 已废弃, 不再对外暴露为特定窗口设置主题的接口
@@ -1045,7 +1109,7 @@ DPalette DGuiApplicationHelper::applicationPalette() const
   的 themeName 和所有与 \a palette 相关的属性都不再生效.
   \warning 使用此方式设置的调色板将不会自动跟随活动色的变化
   \warning 如果使用过QGuiApplication::setPalette, 此方式可能不会生效
-  \a palette 要设置的调色板
+  \param palette 要设置的调色板
  */
 void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
 {
@@ -1080,14 +1144,10 @@ void DGuiApplicationHelper::setApplicationPalette(const DPalette &palette)
   2. 通过 setThemeType 固定调色板的类型
   3. 通过 QGuiApplication::setPalette 固定调色板, 需要注意此方法不可逆.
   否则将基于窗口所对应的 DPlatformTheme 获取调色板( fetchPalette).
-  \a window
+  \param window
   \return 调色板
-  \sa windowTheme
-  \sa fetchPalette
-  \sa standardPalette
-  \sa generatePalette
-  \sa applicationPalette
-  \warning 使用时要同时关注 paletteChanged, 收到此信号后可能需要重新获取窗口的调色板
+  \sa windowTheme(), fetchPalette(), standardPalette(), generatePalette(), applicationPalette()
+  \warning 使用时要同时关注 applicationPaletteChanged() , 收到此信号后可能需要重新获取窗口的调色板
   \warning 已废弃, 不再对外暴露控制窗口级别调色板的接口
  */
 DPalette DGuiApplicationHelper::windowPalette(QWindow *window) const
@@ -1130,7 +1190,7 @@ const DFontManager *DGuiApplicationHelper::fontManager() const
 
   转换的策略为：先将颜色转换为rgb格式，再根据 Y = 0.299R + 0.587G + 0.114B 的公式
   计算出颜色的亮度，亮度大于 191 时认为其为浅色，否则认为其为深色。
-  \a color 需要转换为主题的类型的颜色
+  \param color 需要转换为主题的类型的颜色
   \return 颜色类型的枚举值
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor &color)
@@ -1156,7 +1216,7 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QColor
   使用 QPalette::background 获取颜色的明亮度，将其转换为主题类型的枚举值。
   返回调色板的颜色类型
 
-  \a palette 调色板
+  \param palette 调色板
   \return 颜色类型的枚举值
  */
 DGuiApplicationHelper::ColorType DGuiApplicationHelper::toColorType(const QPalette &palette)
@@ -1204,8 +1264,8 @@ DGuiApplicationHelper::ColorType DGuiApplicationHelper::paletteType() const
 /*!
   \brief 设置DGuiApplicationHelper实例.
 
-  \a key 实例关键字
-  \a singleScope 实例使用范围
+  \param key 实例关键字
+  \param singleScope 实例使用范围
   \return 设置是否成功
   \note 此处所用到DGuiApplicationHelperPrivate::waitTime默认值为3000ms，可通过
   \note DGuiApplicationHelper::setSingleInstanceInterval设置
@@ -1328,7 +1388,7 @@ bool DGuiApplicationHelper::setSingleInstance(const QString &key, DGuiApplicatio
   \brief 设置从QLocalServer获取消息的等待时间.
 
   用于在重新创建DGuiApplicationHelper单例时，检测DGuiApplicationHelper单例是否存在且有响应
-  \a interval 等待时间，如 \a interval 为 -1 则没有超时一直等待，默认和 QLocalSocket 一致 3000ms
+  \param interval 等待时间，如 \a interval 为 -1 则没有超时一直等待，默认和 QLocalSocket 一致 3000ms
   \note 需要在 DGuiApplicationHelper::setSingleInstance 之前调用否则无效。
  */
 void DGuiApplicationHelper::setSingleInstanceInterval(int interval)
@@ -1340,13 +1400,39 @@ void DGuiApplicationHelper::setSingleInstanceInterval(int interval)
 /*!
   \brief 设置从QLocalServer获取消息的等待时间.
 
-  \a interval 等待时间，typo 请使用 DGuiApplicationHelper::setSingleInstanceInterval
+  \param interval 等待时间，typo 请使用 DGuiApplicationHelper::setSingleInstanceInterval
  */
 void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
 {
     DGuiApplicationHelperPrivate::waitTime = interval;
 }
 
+static bool hasLocalManualFile()
+{
+    DCORE_USE_NAMESPACE
+    if (QStandardPaths::findExecutable(QLatin1String("dman")).isEmpty())
+        return false;
+
+    // search all subdirectories
+    const QString xdgDataPath = qgetenv("XDG_DATA_DIRS");
+    auto dataPath = xdgDataPath.split(":");
+    // /usr/share /usr/loacal/share ...
+    const auto &dataDirs = DStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
+    dataPath.append(dataDirs);
+    for (const auto &path : dataPath) {
+        QString strManualPath = QStringList {path, "deepin-manual"}.join(QDir::separator());
+
+        QDirIterator it(strManualPath, QDir::AllDirs | QDir::NoDotAndDotDot,QDirIterator::Subdirectories);
+        while (it.hasNext()) {
+            it.next();
+            if (it.fileName().contains(qApp->applicationName(), Qt::CaseInsensitive))
+                return true;
+        }
+    }
+
+    return false;
+}
+
 /*!
  * \brief Determine whether it's a user manual for this application.
  * \return
@@ -1354,44 +1440,115 @@ void DGuiApplicationHelper::setSingelInstanceInterval(int interval)
 bool DGuiApplicationHelper::hasUserManual() const
 {
 #ifdef Q_OS_LINUX
-    auto loadManualFromLocalFile = [] () -> bool {
-        const QString appName = qApp->applicationName();
-        if (!QFile::exists("/usr/bin/dman"))
-            return false;
-
-        bool dmanDataExists = false;
-        // search all subdirectories
-        QString strManualPath = "/usr/share/deepin-manual";
-        QDirIterator it(strManualPath, QDirIterator::Subdirectories);
-        while (it.hasNext()) {
-            QFileInfo file(it.next());
-            if (file.isDir() && file.fileName().contains(appName, Qt::CaseInsensitive)) {
-                dmanDataExists = true;
-                break;
-            }
-
-            if (file.isDir())
-                continue;
-        }
-        return dmanDataExists;
-    };
+    static qint8 hasManual = -1;
+    if (hasManual >= 0)
+        return hasManual;
 
     QDBusConnection conn = QDBusConnection::sessionBus();
-    if (conn.interface()->isServiceRegistered("com.deepin.Manual.Search")) {
-        QDBusInterface manualSearch("com.deepin.Manual.Search",
-                                    "/com/deepin/Manual/Search",
-                                    "com.deepin.Manual.Search");
-        if (manualSearch.isValid()) {
-            QDBusReply<bool> reply = manualSearch.call("ManualExists", qApp->applicationName());
-            return reply.value();
-        }
+    if (!conn.isConnected()) {
+        static LoadManualServiceWorker *manualWorker = new LoadManualServiceWorker;
+        manualWorker->checkManualServiceWakeUp();
+
+        // 可能存在 dbus 服务有问题，但是文件存在，这时不更新缓存
+        return /*hasManual = */hasLocalManualFile();
     }
-    return loadManualFromLocalFile();
+
+    QDBusInterface manualSearch("com.deepin.Manual.Search",
+                                "/com/deepin/Manual/Search",
+                                "com.deepin.Manual.Search");
+    if (!manualSearch.isValid())
+        return hasManual = hasLocalManualFile();
+
+    QDBusPendingCall call = manualSearch.asyncCall("ManualExists", qApp->applicationName());
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, const_cast<DGuiApplicationHelper *>(this));
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [](QDBusPendingCallWatcher *pWatcher) {
+        QDBusPendingReply<bool> reply = *pWatcher;
+        if (reply.isError()) {
+            qWarning() << reply.error();
+        } else {
+            hasManual = reply.value();
+        }
+
+        pWatcher->deleteLater();
+    });
+
+    return hasManual >= 0 ? hasManual : false;
 #else
     return false;
 #endif
 }
 
+bool DGuiApplicationHelper::loadTranslator(const QString &fileName, const QList<QString> &translateDirs, const QList<QLocale> &localeFallback)
+{
+    DCORE_USE_NAMESPACE;
+
+    QList<QString> dirs = translateDirs;
+    const QList<DPathBuf> defaultDirPrefix {
+        qApp->applicationDirPath(),
+        QDir::currentPath()
+    };
+    for (auto item : defaultDirPrefix)
+        dirs << item.join("translations").toString();
+
+    QStringList missingQmfiles;
+    for (const auto &locale : localeFallback) {
+        QStringList translateFilenames {QString("%1_%2").arg(fileName).arg(locale.name())};
+        const QStringList parseLocalNameList = locale.name().split("_", QString::SkipEmptyParts);
+        if (parseLocalNameList.length() > 0)
+            translateFilenames << QString("%1_%2").arg(fileName).arg(parseLocalNameList.at(0));
+
+        for (const auto &translateFilename : translateFilenames) {
+            for (const auto &dir : dirs) {
+                DPathBuf path(dir);
+                QString translatePath = (path / translateFilename).toString();
+                if (QFile::exists(translatePath + ".qm")) {
+                    qDebug() << "load translate" << translatePath;
+                    auto translator = new QTranslator(qApp);
+                    translator->load(translatePath);
+                    qApp->installTranslator(translator);
+                    qApp->setProperty("dapp_locale", locale.name());
+                    return true;
+                }
+            }
+
+            // fix english does not need to translation.
+            if (locale.language() != QLocale::English)
+                missingQmfiles << translateFilename + ".qm";
+        }
+    }
+
+    if (missingQmfiles.size() > 0) {
+        qWarning() << fileName << "can not find qm files" << missingQmfiles;
+    }
+    return false;
+}
+
+DGuiApplicationHelper::SizeMode DGuiApplicationHelper::sizeMode() const
+{
+    D_DC(DGuiApplicationHelper);
+    return d->fetchSizeMode();
+}
+
+void DGuiApplicationHelper::setSizeMode(const DGuiApplicationHelper::SizeMode mode)
+{
+    D_D(DGuiApplicationHelper);
+    const auto old = d->fetchSizeMode();
+    d->explicitSizeMode = mode;
+    const auto current = d->fetchSizeMode();
+    if (old != current)
+        Q_EMIT sizeModeChanged(current);
+}
+
+void DGuiApplicationHelper::resetSizeMode()
+{
+    D_D(DGuiApplicationHelper);
+    const auto old = d->fetchSizeMode();
+    d->explicitSizeMode = InvalidSizeMode;
+    const auto current = d->fetchSizeMode();
+    if (current != old)
+        Q_EMIT sizeModeChanged(current);
+}
+
 void DGuiApplicationHelper::setAttribute(DGuiApplicationHelper::Attribute attribute, bool enable)
 {
     if (attribute < Attribute::ReadOnlyLimit) {
@@ -1429,10 +1586,8 @@ bool DGuiApplicationHelper::testAttribute(DGuiApplicationHelper::Attribute attri
 
 /*!
   \brief DGuiApplicationHelper::setThemeType.
-  \obsolete
-
-  同 setPaletteType， 已废弃，请不要再使用。
-  \a themeType 主题类型.
+  \deprecated 同 setPaletteType， 已废弃，请不要再使用。
+  \param themeType 主题类型.
  */
 void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeType)
 {
@@ -1448,7 +1603,7 @@ void DGuiApplicationHelper::setThemeType(DGuiApplicationHelper::ColorType themeT
   \note 主动设置调色板颜色类型的操作会导致程序不再使用 DPlatformTheme 中调色板相关的数据, 也
   包括窗口级别的 windowTheme 所对应的 DPlatformTheme, 届时设置 DPlatformTheme
   的 themeName 和所有与 palette 相关的属性都不再生效.
-  \a paletteType 主题类型的枚举值
+  \param paletteType 主题类型的枚举值
  */
 void DGuiApplicationHelper::setPaletteType(DGuiApplicationHelper::ColorType paletteType)
 {
@@ -1484,12 +1639,19 @@ void DGuiApplicationHelper::handleHelpAction()
     QDBusInterface manual("com.deepin.Manual.Open",
                           "/com/deepin/Manual/Open",
                           "com.deepin.Manual.Open");
-    QDBusReply<void> reply = manual.call("ShowManual", appid);
-    if (reply.isValid())  {
-        return;
-    }
-    // fallback to old interface
-    QProcess::startDetached("dman", QStringList() << appid);
+    QDBusPendingCall call = manual.asyncCall("ShowManual", appid);
+    QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(call, this);
+    QObject::connect(watcher, &QDBusPendingCallWatcher::finished, this, [appid](QDBusPendingCallWatcher *pWatcher) {
+        QDBusPendingReply<void> reply = *pWatcher;
+        if (reply.isError()) {
+            // fallback to old interface
+            qWarning() << reply.error() << "fallback to dman appid";
+            QProcess::startDetached("dman", QStringList() << appid);
+        }
+
+        pWatcher->deleteLater();
+    });
+
 #else
     qWarning() << "not support dman now";
 #endif
